#  ---------------------------------------------------------------------
#   Hussein's Picture Archive - OAI 2 data provider
#    v2.05
#    June 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package HSPICS::HSPICSDP;


use Pure::EZXML;
use Pure::X2D;

use OAI::OAI2DP;
use vars ('@ISA');
@ISA = ("OAI::OAI2DP");


# constructor
sub new
{
   my ($classname, $configfile) = @_;
   my $self = $classname->SUPER::new ($configfile);

   # get configuration from file
   my $con = new Pure::X2D ($configfile);
   $self->{'repositoryName'} = $con->param ('repositoryName', 'Picture Archive');
   $self->{'adminEmail'} = $con->param ('adminEmail', "someone\@somewhere");
   $self->{'archiveId'} = $con->param ('archiveId', 'HSPICSArchive');
   $self->{'recordlimit'} = $con->param ('recordlimit', 30);

   $self->{'resumptionseparator'} = '!';
   $self->{'picdir'} = '/home/jcdlpix/public_html/pictures',
   $self->{'urlprefix'} = 'http://rocky.dlib.vt.edu/~jcdlpix/pictures',
   $self->{'publisher'} = 'JCDL';
   $self->{'creator'} = 'Hussein Suleman';
   $self->{'language'} = 'en-us';
   $self->{'relation'} = 'http://www.jcdl.org';
   
   # add in a dummy mdf
   $self->{'metadatanamespace'}->{'dc2'} = 'http://www.openarchives.org/OAI/2.0/oai_dc/';
   $self->{'metadataschema'}->{'dc2'} = 'http://www.openarchives.org/OAI/2.0/oai_dc.xsd';
   $self->{'metadataroot'}->{'dc2'} = 'dc';
   $self->{'metadatarootparameters'}->{'dc2'} = '';

   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->SUPER::dispose ();
}


# datetime functions
sub ToISO
{
   my ($self, $date) = @_;
   my ($day, $month, $year);
   my @monthnames = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 
                     'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

   my @parts = split (' ', $date); # split date

   if ($#parts == -1) { $parts[0] = '01'; } # check for missing fields
   if ($#parts == 0) { $parts[1] = 'Jan'; } # check for missing fields
   if ($#parts == 1) { $parts[2] = '2000'; }

   if ($parts[0] =~ /^[0-9]$/)
   { # check for single digit day number
      $parts[0] = '0'.$parts[0];
   }
   elsif ($parts[0] !~ /^[0-9]{2}$/)
   { # check for double digit day number
      $parts[0] = '01';
   }

   if ($parts[1] !~ /^(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)/i)
   { # check for valid monthname
      $parts[1] = 'Jan';
   }
   my $i;
   for ( $i=0; $i<=$#monthnames; $i++ )
   { # convert name to number
      if ($parts[1] =~ /$monthnames[$i]/i)
      {
         $parts[1] = $i+1; last;
      }
   }
   if ($parts[1] < 10) 
   { # add leading zero
      $parts[1] = '0'.$parts[1];
   }

   if ($parts[2] !~ /[0-9]{4}/)
   { # check for valid y2k year
      $parts[2] = '2000';
   }

   "$parts[2]-$parts[1]-$parts[0]";
}


# special code to read in all data
sub ReadDatabase
{
   my ($self) = @_;

   opendir (DIR, $self->{'picdir'});
   my @dirnames = readdir (DIR);
   close (DIR);

   @dirnames = sort { $b cmp $a } @dirnames;
   pop @dirnames;
   pop @dirnames;

   my @database = ();
 
   my $adir;
   foreach $adir (@dirnames)
   {
      open (FILE, "$self->{picdir}/$adir/descript.txt");
      my @descript = <FILE>;
      close (FILE);
      chomp (@descript);
      unshift (@descript, $adir);
      push (@database, [ @descript ] );
   }

   \@database;
}


# format DC record
sub FormatDC
{
   my ($self, $hashref) = @_;

   {
      title       => $hashref->{'title'},
      creator     => $self->{'creator'},
      subject     => $hashref->{'archivetitle'},
      description => $hashref->{'description'},
      publisher   => $self->{'publisher'},
      date        => $hashref->{'picturedate'},
      type        => 'image',
      format      => 'image/jpeg',
      identifier  => $hashref->{'url'},
      language    => $self->{'language'},
      relation    => $self->{'relation'},
      rights      => 'unrestricted',
      mdorder     => [ qw (title creator subject description contributor publisher date type format identifier source language relation coverage rights) ]
   };
}


# format header for ListIdentifiers
sub Archive_FormatHeader
{
   my ($self, $hashref, $metadataFormat) = @_;
   
   $self->FormatHeader ($hashref->{'urn'},
                        $hashref->{'updatedate'},
                        '',
                        [ $hashref->{'set'} ]
                       );
}


# retrieve records from the source archive as required
sub Archive_FormatRecord
{
   my ($self, $hashref, $metadataFormat) = @_;
   
   if ($self->MetadataFormatisValid ($metadataFormat) == 0)
   {
      $self->AddError ('cannotDisseminateFormat', 'The value of metadataPrefix is not supported by the repository');
      return '';
   }

   # perform a little bit of pre-cleaning of fields
   $hashref->{'title'}        = $self->{'utility'}->lclean ($hashref->{'title'});
   $hashref->{'archivetitle'} = $self->{'utility'}->lclean ($hashref->{'archivetitle'});
   $hashref->{'description'}  = $self->{'utility'}->pclean ($hashref->{'description'});
   $hashref->{'picturedate'}  = $self->{'utility'}->lclean ($hashref->{'picturedate'});
   $hashref->{'updatedate'}   = $self->{'utility'}->lclean ($hashref->{'updatedate'});
   $hashref->{'url'}          = $self->{'utility'}->lclean ($hashref->{'url'});

   my $dc = $self->FormatDC ($hashref);
   
   my $header = "<oaidc:dc xmlns=\"http://purl.org/dc/elements/1.1/\" ".
                "xmlns:oaidc=\"http://www.openarchives.org/OAI/2.0/oai_dc/\" ".
                "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ".
                "xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai_dc/ ".
                "http://www.openarchives.org/OAI/2.0/oai_dc.xsd\">\n";
   my $footer = "</oaidc:dc>\n";

   $self->FormatRecord ($hashref->{'urn'},
                        $hashref->{'updatedate'},
                        '',
                        [ $hashref->{'set'} ],
                        $header.$self->{'utility'}->FormatXML ($dc).$footer,
                        '',
                       );
}


# get full list of mdps or list for specific identifier
sub Archive_ListMetadataFormats
{
   my ($self, $identifier) = @_;
   
   if ((! defined $identifier) || ($identifier eq ''))
   {
      return [];
   }
   
   my @fulldata = split (':', $identifier);
   if ($#fulldata < 2)
   {
      $self->AddError ('badArgument', 'Illegal format for identifier argument');
      return;
   }

   my $database = $self->ReadDatabase ();

   my $adir = $fulldata[2];
   $adir =~ s/([0-9]{4}[0-9]{2}[a-z]+)[0-9]+/$1/;
   my $filenumber = $fulldata[2];
   $filenumber =~ s/([0-9]{4}[0-9]{2}[a-z]+)([0-9]+)/$2/;
   my $filename = $fulldata[2];
   $filename =~ s/[0-9]{2}([0-9]{2})[0-9]{2}([a-z]+[0-9]+)/$1$2/;

   my $found = 0;
   foreach my $arecord (@$database)
   {
      if (($$arecord[0] eq $adir) && ($filenumber <= $$arecord[5]))
      { 
         $found = 1; 
         last; 
      }
   }
   if ($found == 0)
   {
      $self->AddError ('idDoesNotExist', 'The value of the identifier argument is unknown or illegal in this repository');
   }
   return [];
}


# get full list of sets from the archive
sub Archive_ListSets
{
   my ($self) = @_;
   
   my $database = $self->ReadDatabase ();
      
   my $setlist = [];
   
   my $arecord;
   foreach $arecord (@$database)
   {
      push (@$setlist, [ $$arecord[0], 
                         $$arecord[1],
                         "<oaidc:dc xmlns:oaidc=\"http://www.openarchives.org/OAI/2.0/oai_dc/\" ".
                         "xmlns=\"http://purl.org/dc/elements/1.1/\" ".
                         "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ".
                         "xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai_dc/ ".
                         "http://www.openarchives.org/OAI/2.0/oai_dc.xsd\">\n".
                         "  <description>Pictures taken during $$arecord[1]</description>\n".
                         "</oaidc:dc>\n"
                       ]);
   }
                           
   $setlist;
}
                              

sub Archive_GetRecord
{
   my ($self, $identifier, $metadataFormat) = @_;

   my @fulldata = split (':', $identifier);

   my $database = $self->ReadDatabase ();

   my $adir = $fulldata[2];
   $adir =~ s/([0-9]{4}[0-9]{2}[a-z]+)[0-9]+/$1/;
   my $filenumber = $fulldata[2];
   $filenumber =~ s/([0-9]{4}[0-9]{2}[a-z]+)([0-9]+)/$2/;
   my $filename = $fulldata[2];
   $filename =~ s/[0-9]{2}([0-9]{2})[0-9]{2}([a-z]+[0-9]+)/$1$2/;

   my $arecord;
   my $record;
   my $found = 0;
   foreach $arecord (@$database)
   {
      if (($$arecord[0] eq $adir) && ($filenumber <= $$arecord[5]))
      { 
         $found = 1; 
         $record = $arecord;
         last; 
      }
   }
   if ($found == 1)
   {
      {
         urn                => $identifier,
         title              => $filename,
         archivetitle       => $$record[1],
         picturedate        => $self->ToISO ($$record[2]),
         archivedescription => $$record[3],
         updatedate         => $self->ToISO ($$record[4]),
         description        => $$record[$filenumber+5],
         url                => "$self->{urlprefix}/$adir/$filename.jpg",
         set                => $$record[0],
      };
   }
   else
   {
      $self->AddError ('idDoesNotExist', 'The value of the identifier argument is unknown or illegal in this repository');
      undef;
   }
}

sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   my ($offset);
   if ($resumptionToken eq '')
   {
      $offset = 0;
   }
   else
   {
      my @rdata = split ($self->{'resumptionseparator'}, $resumptionToken);
      ($set, $from, $until, $metadataPrefix, $offset) = @rdata;
   }

   my $database = $self->ReadDatabase ();

   my $arecord;
   my $count = 0;
   my @allrows = ();
   my $gotmore = 0;
   foreach $arecord (@$database)
   {
      if ((($set eq '') || ($$arecord[0] eq $set)) &&
          (($from eq '') || ($self->ToSeconds ($self->ToISO ($$arecord[4])) >= $self->ToSeconds ($from, 1))) &&
          (($until eq '') || ($self->ToSeconds ($self->ToISO ($$arecord[4])) <= $self->ToSeconds ($until))))
      {
         my $i;
         for ( $i=1; $i<=$$arecord[5]; $i++ )
         {
            $count++;
            if ($count > $offset)
            {
               if ($count <= $offset+$self->{'recordlimit'})
               {
                   my $filename = $$arecord[0];
                   $filename =~ s/[0-9]{2}([0-9]{2})[0-9]{2}([a-z]+)/$1$2/;
                   my $rowref = { 
                     urn                => 'oai:'.$self->{'archiveId'}.':'.$$arecord[0].$i,
                     title              => $filename,
                     archivetitle       => $$arecord[1],
                     picturedate        => $self->ToISO ($$arecord[2]),
                     archivedescription => $$arecord[3],
                     updatedate         => $self->ToISO ($$arecord[4]),
                     description        => $$arecord[$i+5],
                     url                => "$self->{'urlprefix'}/$$arecord[0]/$filename$i.jpg",
                     set                => $$arecord[0],
                  };
                  push (@allrows, $rowref);
               }
               else
               {
                  $gotmore = 1;
               }
            }
         }
      }
   }

   $resumptionToken = '';
   if ($gotmore == 1)
   {
      $resumptionToken = join ($self->{'resumptionseparator'}, ($set,$from,$until,$metadataPrefix,$offset+$self->{'recordlimit'}));
   }
   if ($count == 0)
   {
      $self->AddError ('noRecordsMatch', 'The combination of the values of arguments results in an empty set');
   }

   ( \@allrows, $resumptionToken, $metadataPrefix, { 'completeListSize' => $count, 'cursor' => $offset } );
}

sub Archive_ListIdentifiers
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   if (($metadataPrefix ne '') && ($self->MetadataFormatisValid ($metadataPrefix) == 0))
   {
      $self->AddError ('cannotDisseminateFormat', 'The value of metadataPrefix is not supported by the repository');
      return '';
   }
   
   $self->Archive_ListRecords ($set, $from, $until, $metadataPrefix, $resumptionToken);
}


1;

